/*
 * C
 *
 * Copyright 2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

/* Includes ------------------------------------------------------------------*/

#include "LLKERNEL_impl.h"
#include "BESTFIT_ALLOCATOR.h"
#include <stdio.h>
#include <string.h>
#include "bsp_util.h"
#include "sni.h"
#include "fsl_debug_console.h"

/* Macros and defines --------------------------------------------------------*/

// declare MicroEJ KF Working buffer
BSP_DECLARE_BUFFER(KERNEL_WORKING_BUFFER)

/* Globals -------------------------------------------------------------------*/

BESTFIT_ALLOCATOR allocatorInstance;
uint32_t KERNEL_allocationInitialized;

/* API -----------------------------------------------------------------------*/

void* LLKERNEL_IMPL_allocate(int32_t size){
	if(KERNEL_allocationInitialized == 0){
		// lazy init
		BESTFIT_ALLOCATOR_new(&allocatorInstance);
		BESTFIT_ALLOCATOR_initialize(&allocatorInstance, KERNEL_WORKING_BUFFER_START, KERNEL_WORKING_BUFFER_END);
		KERNEL_allocationInitialized = 1;
	}

	void* allocationStart = BESTFIT_ALLOCATOR_allocate(&allocatorInstance, size);
#ifdef ALLOCATOR_DEBUG
	PRINTF("LLKERNEL_IMPL_allocate %d %p \n", size, allocationStart);
#endif
	return (void*)allocationStart;
}

void LLKERNEL_IMPL_free(void* block){
	BESTFIT_ALLOCATOR_free(&allocatorInstance, block);
#ifdef ALLOCATOR_DEBUG
	PRINTF("LLKERNEL_IMPL_free %p \n", block);
#endif
}

/* Simple Kernel -------------------------------------------------------------*/
