/*
 * C
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
#if !defined DISPLAY_TASK_H
# define DISPLAY_TASK_H

// -----------------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------------

#include "vglite_window.h"

#include "mej_debug.h"

// -----------------------------------------------------------------------------
// Macros and Defines
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Types
// -----------------------------------------------------------------------------

typedef void (* display_task_flush_callback_t) (void * params);

// -----------------------------------------------------------------------------
// Variables
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Public functions
// -----------------------------------------------------------------------------

/*
 * Initialize the display task
 * @param window: the VGLite window of the display
 */
void display_task_initialize(vg_lite_window_t *window);

/*
 * @brief: Wake-up display task to launch a flush or to wait the end of previous VG drawing
 *
 * @param[in] flush_or_vg:
 * 	- if true: Wake up the display task for to launch a flush,
 * 	- otherwise: Wake up the display task to wait a VG drawing.
 */
void display_task_wakeup(bool flush_or_vg);


/*
 * flush given address to the display
 * @param[in] sourceAddr the address of source buffer (current graphics buffer)
 * @param[in] xmin the top-left X-coordinate of the dirty area.
 * @param[in] ymin the top-left Y-coordinate of the dirty area.
 * @param[in] xmax the bottom-right X-coordinate of the dirty area.
 * @param[in] ymax the bottom-right Y-coordinate of the dirty area.
 */
void display_task_flush(uint8_t* addr, uint32_t xmin, uint32_t ymin, uint32_t xmax, uint32_t ymax);

/*
 * @brief Registers a callback to perform extra flush operations
 *
 * @param[in] flush_callback: Pointer to the callback function
 */
void display_task_register_flush_callback(display_task_flush_callback_t callback);

#endif // !defined DISPLAY_TASK_H

// -----------------------------------------------------------------------------
// EOF
// -----------------------------------------------------------------------------

