/*
 * C
 *
 * Copyright 2019-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

#if !defined __MEJ_DISABLE_H__
# define __MEJ_DISABLE_H__

/*
 * @brief GPU is less efficient than CPU to perform simple aliased drawings (line, rectangle etc.)
 *
 * This define forces to use software algorithms. Comment it to use GPU instead.
 */
#define MEJ_DISABLED_FOR_PERFORMANCE_SIMPLE_DRAWINGS

/*
 * @brief GPU is less efficient than CPU to perform a simple "draw image": when the image to render has
 * the same pixel definition than the destination buffer and no alpha blending is required.
 *
 * This defines forces to use software algorithm to draw RGB565 images. Comment it to use GPU instead.
 */
#define MEJ_DISABLED_FOR_PERFORMANCE_IMAGE_RGB565

/*
 * @brief GCNanoLite-V does not support MSAA. By consequence the "draw image" functions (with or without
 * transformation like rotation or scale) cannot use GPU when the image to render contains transparent
 * pixels.
 *
 * This define forces to use software algorithms instead. Comment it to use GPU instead.
 */
#define MEJ_DISABLED_FOR_MSAA_LIMITATION

/*
 * @brief Display tearing signal prevents glitches, however it reduces the maximum frame per seconds.
 *
 * This define disables the use of tearing. Comment it to enable tearing.
 */
#define MEJ_DISABLED_DISPLAY_TEARING

#endif // __MEJ_DISABLE_H__

