/*
* Copyright 2018, Silicon Laboratories Inc.  All rights reserved.
* 
* Copyright 2019-2021 MicroEJ Corp. This file has been modified by MicroEJ Corp.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

/**************************************************************************//**
 * Host specific implementation: LPC546XX + FreeRTOS
 *****************************************************************************/

#pragma once
#include <stdint.h>
#include "sl_wfx.h"
#include "FreeRTOS.h"
#include "event_groups.h"

#define SOFTAP_MODE 1
#define STATION_MODE 0

extern struct netif sta_gnetif;
extern struct netif ap_gnetif;


/* WFX host callbacks */
void sl_wfx_connect_callback( uint8_t* mac, uint32_t status );
void sl_wfx_disconnect_callback( uint8_t* mac, uint16_t reason );
void sl_wfx_start_ap_callback( uint32_t status );
void sl_wfx_stop_ap_callback( void );
void sl_wfx_host_received_frame_callback( sl_wfx_received_ind_t* rx_buffer );
void sl_wfx_scan_result_callback( sl_wfx_scan_result_ind_body_t* scan_result );
void sl_wfx_scan_complete_callback( uint32_t status );
void sl_wfx_generic_status_callback( sl_wfx_generic_ind_t* frame );
void sl_wfx_client_connected_callback( uint8_t* mac );
void sl_wfx_ap_client_disconnected_callback( uint32_t status, uint8_t* mac );

void SPI_WIRQ_INTERRUPT_HANDLER(void);
