/*
 * Java
 *
 * Copyright 2021 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.kernel;

import com.microej.demo.watch.util.WatchfaceApp;

import ej.kf.Feature;
import ej.kf.Feature.State;
import ej.kf.FeatureStateListener;
import ej.kf.Kernel;

/**
 * Watchfaces registry using sandboxed applications.
 */
public class DynamicWatchfacesRegistry extends WatchfacesRegistry {

	private FeatureStateListener featureStateListener;

	/**
	 * Creates a dynamic watchfaces registry.
	 */
	public DynamicWatchfacesRegistry() {
		this.featureStateListener = new FeatureStateListener() {
			@Override
			public void stateChanged(Feature feature, State previousState) {
				State newState = feature.getState();
				if (newState == State.UNINSTALLED) {
					removeFeatureWatchfaces(feature);
				}
			}
		};

		Kernel.addFeatureStateListener(this.featureStateListener);
	}

	/**
	 * Disposes this dynamic watchfaces registry.
	 */
	public void dispose() {
		if (this.featureStateListener != null) {
			Kernel.removeFeatureStateListener(this.featureStateListener);
			this.featureStateListener = null;
		}
	}

	private void removeFeatureWatchfaces(Feature feature) {
		for (int i = 0; i < getNumWatchfaces(); i++) {
			WatchfaceApp watchface = getWatchface(i);
			if (Kernel.getOwner(watchface) == feature) {
				removeWatchface(watchface);
			}
		}
	}
}
