/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.path.vectorfont;

import com.microej.demo.watch.util.path.GradientPathGraphics;
import com.microej.demo.watch.util.path.GradientStyle;
import com.microej.vivante.utils.GradientPath;
import com.verisilicon.vivante.vg_lite;
import com.verisilicon.vivante.vg_lite_gradient;
import com.verisilicon.vivante.vg_lite_matrix;
import com.verisilicon.vivante.vg_lite_path;

/**
 * A vector graphics that represents an hexagon.
 */
public class Hexagon extends GradientPathGraphics {

	private static final int GRADIENT_SCALE = 2;

	private static final int TRANSLATE = 50;

	private static final int PATH_BOUND_WIDTH = 100;

	private static final int PATH_BOUND_HEIGHT = 100;

	private static final byte[] PATH_ARRAY = { 2, 75, 97, 8, (byte) 68.504808, 100, (byte) 31.495188, 100, 25, 98, 8,
			(byte) 18.504807, 100, 0, (byte) (255.13205 - 197), 0, 50, 8, 0, (byte) (238.86794 - 197), (byte) 18.504811,
			(byte) (204.11555 - 197), 25, 3, 8, (byte) 31.495193, 0, (byte) 68.504812, 0, 75, 3, 8, (byte) 81.495193,
			(byte) (204.11555 - 197), 100, (byte) (238.86794 - 197), 100, 50, 8, 100, (byte) (255.13206 - 197),
			(byte) 81.495189, (byte) (289.88444 - 197), 75, 97, 0 };

	private float translateY;

	private float scale;

	private float translateX;

	/**
	 * Creates an hexagon, using the specified gradient information .
	 *
	 * @param gradientStyle
	 *            the gradient information.
	 *
	 */
	public Hexagon(GradientStyle gradientStyle) {
		super(createPath(), gradientStyle);
		initializeGradient(gradientStyle.getAngle());
	}

	private void initializeGradient(float angle) {
		final vg_lite_matrix gradientMatrix = getGradient().getMatrix();
		vg_lite_matrix.vg_lite_identity(gradientMatrix);
		vg_lite_matrix.vg_lite_translate(0, TRANSLATE, gradientMatrix);
		vg_lite_matrix.vg_lite_scale(GRADIENT_SCALE, GRADIENT_SCALE, gradientMatrix);
		vg_lite_matrix.vg_lite_rotate(angle, gradientMatrix);
	}

	/**
	 * @return
	 */
	private static GradientPath createPath() {
		return new GradientPath(new vg_lite_path(0, 0, PATH_BOUND_WIDTH, PATH_BOUND_HEIGHT, vg_lite_path.VG_LITE_HIGH,
				vg_lite_path.VG_LITE_S8, PATH_ARRAY), new vg_lite_gradient(), vg_lite.VG_LITE_FILL_EVEN_ODD);
	}

	@Override
	protected void updateMatrices() {
		vg_lite_matrix matrix = getMatrix();
		vg_lite_matrix.vg_lite_identity(matrix);
		vg_lite_matrix.vg_lite_translate(this.translateX, this.translateY, matrix);
		vg_lite_matrix.vg_lite_scale(this.scale, this.scale, matrix);
		vg_lite_matrix.vg_lite_translate(-TRANSLATE, -TRANSLATE, matrix);
	}

	/**
	 * Sets the transformation to apply to this vector graphics.
	 *
	 * @param translateX
	 *            the horizontal translation to apply.
	 * @param translateY
	 *            the vertical translation to apply.
	 * @param scale
	 *            the scale to apply.
	 */
	public void setTransformation(float translateX, float translateY, float scale) {
		this.translateX = translateX;
		this.translateY = translateY;
		this.scale = scale;
		updateMatrices();
	}
}
