/*
 * Java
 *
 * Copyright 2021 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.service;

import java.io.IOException;
import java.io.InputStream;

import com.microej.demo.watch.util.KernelServiceRegistry;
import com.microej.demo.watch.util.services.FeatureLoaderService;
import com.microej.kernel.FeatureInputStream;

import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.kf.AlreadyLoadedFeatureException;
import ej.kf.Feature;
import ej.kf.IncompatibleFeatureException;
import ej.kf.InvalidFormatException;
import ej.observable.KernelSimpleObservable;

/**
 * Represents the dynamic feature loader.
 */
public class FeatureLoaderServiceImpl extends KernelSimpleObservable implements FeatureLoaderService {

	private static final int PERIOD = 2_000;

	private TimerTask timerTask;

	/**
	 * Creates the feature loader service.
	 */
	public FeatureLoaderServiceImpl() {
		FeatureInputStream.init();
	}

	@Override
	public void start() {
		stop();
		this.timerTask = new TimerTask() {
			@Override
			public void run() {
				try (InputStream is = FeatureInputStream.getInputStream()) {
					if (null != is) {
						System.out.println("Application available. Loading application."); //$NON-NLS-1$
						Feature feature = ej.kf.Kernel.install(is);
						feature.start();
					}
				} catch (IOException | InvalidFormatException | IncompatibleFeatureException
						| AlreadyLoadedFeatureException e) {
					System.out.println("Application loading error:"); //$NON-NLS-1$
					e.printStackTrace();
				}

			}
		};

		System.out.println("Waiting for an application to be available"); //$NON-NLS-1$
		Timer timer = KernelServiceRegistry.getServiceLoader().getService(Timer.class);
		timer.schedule(this.timerTask, PERIOD, PERIOD);
	}

	@Override
	public void stop() {
		TimerTask task = this.timerTask;
		if (task != null) {
			task.cancel();
			this.timerTask = null;
		}
	}

}
