/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.annotation.processor.model;


/**
 * An interface that allow implementations to be written in GIML (JS code) =>
 * All methods take no arguments and return <code>void</code>
 */
public class CodeInterface {

	private final String fullyQualifiedName;
	private final CodeMethodPrototype[] methods;

	public static class CodeMethodPrototype {

		private final String methodName;

		public CodeMethodPrototype(String methodName) {
			this.methodName = methodName;
		}

		/**
		 * Gets the methodName.
		 *
		 * @return the methodName.
		 */
		public String getName() {
			return methodName;
		}
	}

	/**
	 *
	 */
	public CodeInterface(String fullyQualifiedName, CodeMethodPrototype[] methods) {
		this.fullyQualifiedName = fullyQualifiedName;
		this.methods = methods;
	}

	/**
	 * Gets the methods.
	 *
	 * @return the methods.
	 */
	public CodeMethodPrototype[] getMethods() {
		return methods;
	}

	/**
	 * Gets the fullyQualifiedName of the type of this interface
	 *
	 * @return the fullyQualifiedName.
	 */
	public String getFullyQualifiedName() {
		return fullyQualifiedName;
	}

	/**
	 * Gets the simple name of this interface
	 * @return the simple name of this interface
	 */
	public String getSimpleName() {
		int lastDotIndex = fullyQualifiedName.lastIndexOf('.');
		if(lastDotIndex == -1) {
			return fullyQualifiedName;
		}
		else {
			return fullyQualifiedName.substring(lastDotIndex + 1, fullyQualifiedName.length());
		}
	}
}
