/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.binary.test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.stringtemplate.v4.ST;
import org.stringtemplate.v4.STGroup;
import org.stringtemplate.v4.STGroupFile;

import com.is2t.giml.binary.GIMLWriter;
import com.is2t.giml.binary.GIMLWriterCompacted;
import com.is2t.giml.generator.GIMLGenerator;

/**
 * Just generate a test file
 */
public class GIMLWriterCompactedGenerateTestFile {

	private static final String FILENAME = "../GIMLRuntime/src/test/resources/com/is2t/giml/runtime/reader/test/testfile.giml";
	private static final String ROOT_ID_INTERFACE = "../GIMLRuntime/src/test/java/com/is2t/giml/runtime/reader/test/RootIds.java";
	private static final String ROOT_ID_INTERFACE_PACKAGE = "com.is2t.giml.runtime.reader.test";
	private static final String GIML_STG_FILE = "com/is2t/giml/generator/giml.stg";
	private static final STGroup GimlSTGroup;
	static{
		GimlSTGroup = new STGroupFile(GIML_STG_FILE);
	}
	
	public static void main(String[] args) throws IOException {
		GIMLWriterCompactedTest generator = new GIMLWriterCompactedTest();
		generator.initWriter(new GIMLWriterCompacted(new FileOutputStream(FILENAME)));

		generator.writeVarUInt();
		generator.writeVarSInt();
		generator.writeVarLong();
		generator.writeNLSStringId();
		generator.writeNonNLSString("yahoo!", "yop", "This is a very long string. This is a very long string. This is a very long string. This is a very long string. This is a very long string. ");
		generator.writeNonNLSUTF8String();
		generator.writeFloat();
		generator.writeDouble();
		generator.writeBoolean();
		markPosition(generator);

		generator.completeWriter(0);

	}

	public static void markPosition(GIMLWriterCompactedTest generator) throws IOException{
		Map<String, Integer> idKeysValues = new HashMap<String, Integer>();
		
		GIMLWriter writer = generator.writer;
		writer.writeVarUInt(42);
		
		idKeysValues.put("id1", 0);
		writer.markPosition(0);
		writer.writeVarUInt(91);
		
		idKeysValues.put("id2", 1);
		writer.markPosition(1);
		writer.writeVarUInt(92);
		
		idKeysValues.put("id3", 3);
		writer.markPosition(3);
		writer.writeVarUInt(93);
		
		idKeysValues.put("id4", 4);
		writer.markPosition(4);
		writer.writeVarUInt(94);
		
		idKeysValues.put("id5", 5);
		writer.markPosition(5);
		writer.writeVarUInt(95);
		
		idKeysValues.put("id6", 6);
		writer.markPosition(6);
		writer.writeVarUInt(96);
		
		idKeysValues.put("id7", 7);
		writer.markPosition(7);
		writer.writeVarUInt(97);
		
		idKeysValues.put("id8", 8);
		writer.markPosition(8);
		writer.writeVarUInt(98);
		
		idKeysValues.put("id9", 9);
		writer.markPosition(9);
		writer.writeVarUInt(99);
		
		idKeysValues.put("id10", 10);
		writer.markPosition(10);
		writer.writeVarUInt(100);
		generateRootIdsInterface(idKeysValues);
	}
	
	private static void generateRootIdsInterface(Map<String, Integer> idKeysValues) throws IOException{
		ST rootIdInterface = GimlSTGroup.getInstanceOf("GIMLRootIdsInterface");
		rootIdInterface.add("interfaceName", "RootIds");
		rootIdInterface.add("package", ROOT_ID_INTERFACE_PACKAGE);
		File rootIdsInterfaceFile = new File(ROOT_ID_INTERFACE);
		rootIdInterface.add("idKeysValues", idKeysValues);
		GIMLGenerator.writeGeneratedGIMLCode(rootIdInterface, rootIdsInterfaceFile);
		
	}

}
