/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.ant;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

import com.is2t.giml.cssgenerator.CSSGenerator;

/**
 * The class {@code CSSGeneratorTask} is the Ant invocation of {@link CSSGenerator}.
 */
public class CSSGeneratorTask extends Task {

	private final List<File> _cssFiles; // NOSONAR
	private String _generatedPackageName; // NOSONAR
	private String _generatedClassName; // NOSONAR
	private String _generatedMethodName; // NOSONAR
	private File _outputDirectory; // NOSONAR

	public CSSGeneratorTask() {
		this._cssFiles = new ArrayList<File>();
	}

	@Override
	public void execute() throws BuildException {
		super.execute();

		// Check all files exists
		for (File cssFile : this._cssFiles) {
			if (!cssFile.exists()) {
				throw new BuildException(String.format("File %s does not exist", cssFile.getAbsolutePath()));
			}
			if (!cssFile.isFile()) {
				throw new BuildException(String.format("%s does not denote a file path", cssFile.getAbsolutePath()));
			}
		}

		// Check output directory is set
		if (this._outputDirectory == null) {
			throw new BuildException("outputDir must be set.");
		}

		// Parse and generate
		try {
			CSSGenerator generator = new CSSGenerator();
			if (this._generatedPackageName != null) {
				generator.setGeneratedPackageName(this._generatedPackageName);
			}
			if (this._generatedClassName != null) {
				generator.setGeneratedClassName(this._generatedClassName);
			}
			if (this._generatedMethodName != null) {
				generator.setGeneratedMethodName(this._generatedMethodName);
			}
			for (File cssFile : this._cssFiles) {
				generator.parseFile(cssFile);
			}
			generator.generate(this._outputDirectory);
		} catch (Exception e) {
			throw new BuildException(e);
		}
	}

	public void setGeneratedPackageName(String name) {
		this._generatedPackageName = name;
	}

	public void setGeneratedClassName(String name) {
		this._generatedClassName = name;
	}

	public void setGeneratedMethodName(String name) {
		this._generatedMethodName = name;
	}

	public void setOutpurDir(File outputDir) {
		this._outputDirectory = outputDir;
	}

	public void setCSSFilesRef(Reference reference) {
		Path dependenciesAsPath = new Path(getProject());
		dependenciesAsPath.setRefid(reference);
		for (String path : dependenciesAsPath.list()) {
			this._cssFiles.add(new File(path));
		}
	}

}
