/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.IOException;
import java.io.StringWriter;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import org.w3c.css.sac.AttributeCondition;
import org.w3c.css.sac.Condition;
import org.w3c.css.sac.SelectorList;

import com.is2t.giml.cssgenerator.util.CSSParserUtils;
import com.steadystate.css.parser.selectors.PseudoClassConditionImpl;

/**
 *
 */
public class PseudoClassSelector implements Selector {

	private static final String STATE_ENUM_NAME = "ej.style.State";
	private static final String EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME = "ej.style.selector.StateSelector";
	private static final String NTH_CHILD_SELECTOR_SELECTOR = "nth-child([0-9]+)";
	private static final String NOT_SELECTOR = "not(.*)";
	private static final String NTH_CHILD_EVEN_SELECTOR = "nth-child(even)";
	private static final String FIRST_CHILD_SELECTOR = "first-child";
	private static final String LAST_CHILD_SELECTOR = "last-child";
	private static final String STATE_SELECTOR_VAR_PREFIX = "state";
	private static final String NOT_SELECTOR_VAR_PREFIX = "not_";
	private static final String NTH_CHILD_SELECTOR_SELECTOR_VAR_PREFIX = "nthChild_";
	private static final String NTH_CHILD_EVEN_SELECTOR_VAR_PREFIX = "nthChild_even";
	private static final String NTH_CHILD_ODD_SELECTOR_VAR_PREFIX = "nthChild_odd";
	private static final String FIRST_CHILD_SELECTOR_VAR_PREFIX = "firstChild";
	private static final String LAST_CHILD_SELECTOR_VAR_PREFIX = "lastChild";
	private static final Map<String, String> PSEUDO_CLASS_MAPPING = new HashMap<String, String>();
	private final Map<String, String> pseudoSelectorVars = new Hashtable<String, String>();

	static {
		PSEUDO_CLASS_MAPPING.put("nth-child(even)", "ej.style.selector.EvenChildSelector");
		PSEUDO_CLASS_MAPPING.put("first-child", "ej.style.selector.FirstChildSelector");
		PSEUDO_CLASS_MAPPING.put("last-child", "ej.style.selector.LastChildSelector");
		PSEUDO_CLASS_MAPPING.put("nth-child([0-9]+)", "ej.style.selector.NthChildSelector");
		PSEUDO_CLASS_MAPPING.put("nth-child(odd)", "ej.style.selector.OddChildSelector");
		PSEUDO_CLASS_MAPPING.put(NOT_SELECTOR, "ej.style.selector.NotSelector");

		PSEUDO_CLASS_MAPPING.put("active", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("checked", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("disabled", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("empty", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("enabled", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("focus", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("hover", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("in-range", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("invalid", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("link", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("optional", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("out-of-range", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("read-only", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("read-write", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("required", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("root", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("target", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("valid", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
		PSEUDO_CLASS_MAPPING.put("visited", EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME);
	}

	private final String _pseudoClassName; // NOSONAR
	private final boolean _isStateSelector;

	public PseudoClassSelector(String pseudoClassName) {
		this._pseudoClassName = pseudoClassName;
		this._isStateSelector = EJ_STYLE_SELECTOR_STATE_SELECTOR_CLASS_NAME
				.equals(PSEUDO_CLASS_MAPPING.get(pseudoClassName));
	}

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		if (!this.matches(c)) {
			throw new IllegalArgumentException("Given condition is not an AttributeCondition");
		}
		AttributeCondition condition = (AttributeCondition) c;

		if (this._isStateSelector) {
			String stateSelectorVar = this.pseudoSelectorVars.get(this._pseudoClassName);
			if (stateSelectorVar != null) {
				return stateSelectorVar;
			}
			StringBuilder stateSelectorVarSb = new StringBuilder(STATE_SELECTOR_VAR_PREFIX);
			String value = this._pseudoClassName;
			value = value.substring(0, 1).toUpperCase() + value.substring(1);
			while (value.contains("-")) {
				int indexOf = value.indexOf('-');
				value = value.substring(0, indexOf) + value.substring(indexOf + 1, indexOf + 2).toUpperCase()
						+ value.substring(indexOf + 2);
			}
			stateSelectorVarSb.append(value);
			writer.append(this.getClassName()).append(" ");
			writer.append(stateSelectorVarSb);
			writer.append(" = new ").append(this.getClassName()).append("(");
			writer.append(STATE_ENUM_NAME).append('.');
			writer.append(value).append(");\n");
			this.pseudoSelectorVars.put(this._pseudoClassName, stateSelectorVarSb.toString());
			return stateSelectorVarSb.toString();
		} else if (condition.getValue().matches(NOT_SELECTOR.replace("(", "\\(").replace(")", "\\)"))) {
			String value = condition.getValue();
			value = value.substring(4, value.length());
			value = value.substring(0, value.length() - 1);
			SelectorList selectors;
			try {
				selectors = CSSParserUtils.getSelectors(value);
				if (selectors.getLength() != 1) {
					throw new IllegalArgumentException(
							String.format("Cannot have multiple selectors in :not() (found %s)", value));
				}
				StringBuilder notSelector = new StringBuilder(this._pseudoClassName);
				String selectorVar = SelectorFactory.create(selectors.item(0), writer);
				notSelector.append(selectorVar);
				String notSelectorVar = this.pseudoSelectorVars.get(notSelector.toString());
				if (notSelectorVar != null) {
					return notSelectorVar;
				}
				StringBuilder notSelectorVarSb = new StringBuilder(NOT_SELECTOR_VAR_PREFIX).append(selectorVar);
				writer.append(this.getClassName()).append(" ").append(notSelectorVarSb);
				writer.append(" = new ").append(this.getClassName()).append("(").append(selectorVar).append(");\n");
				this.pseudoSelectorVars.put(notSelector.toString(), notSelectorVarSb.toString());
				return notSelectorVarSb.toString();

			} catch (IOException e) {
				throw new IllegalArgumentException(e);
			}
		} else if (condition.getValue().matches(NTH_CHILD_SELECTOR_SELECTOR.replace("(", "\\(").replace(")", "\\)"))) {
			String conditionValue = condition.getValue();
			String nthChildSelectorVar = this.pseudoSelectorVars.get(conditionValue);
			if (nthChildSelectorVar != null) {
				return nthChildSelectorVar;
			}
			String value = conditionValue;
			value = value.substring(10, value.length());
			value = value.substring(0, value.length() - 1);

			StringBuilder nthChildSelectorVarSb = new StringBuilder(NTH_CHILD_SELECTOR_SELECTOR_VAR_PREFIX)
					.append(value);
			writer.append(this.getClassName()).append(" ").append(nthChildSelectorVarSb);
			writer.append(" = new ").append(this.getClassName()).append("(").append(value).append(");\n");
			this.pseudoSelectorVars.put(conditionValue, nthChildSelectorVarSb.toString());
			return nthChildSelectorVarSb.toString();
		} else {
			String childSelectorVar = this.pseudoSelectorVars.get(this._pseudoClassName);
			if (childSelectorVar != null) {
				return childSelectorVar;
			}
			StringBuilder childSelectorVarSb = new StringBuilder();
			if (condition.getValue().matches(NTH_CHILD_EVEN_SELECTOR.replace("(", "\\(").replace(")", "\\)"))) {
				childSelectorVarSb.append(NTH_CHILD_EVEN_SELECTOR_VAR_PREFIX);
			} else if (condition.getValue().matches(FIRST_CHILD_SELECTOR.replace("(", "\\(").replace(")", "\\)"))) {
				childSelectorVarSb.append(FIRST_CHILD_SELECTOR_VAR_PREFIX);
			} else if (condition.getValue().matches(LAST_CHILD_SELECTOR.replace("(", "\\(").replace(")", "\\)"))) {
				childSelectorVarSb.append(LAST_CHILD_SELECTOR_VAR_PREFIX);
			} else {
				childSelectorVarSb.append(NTH_CHILD_ODD_SELECTOR_VAR_PREFIX);
			}
			writer.append(this.getClassName()).append(" ").append(childSelectorVarSb);
			writer.append(" = new ").append(this.getClassName()).append("();\n");
			this.pseudoSelectorVars.put(this._pseudoClassName, childSelectorVarSb.toString());
			return childSelectorVarSb.toString();
		}

	}

	@Override
	public boolean matches(Condition condition) {
		if (!(condition instanceof PseudoClassConditionImpl)) {
			return false;
		}

		PseudoClassConditionImpl c = (PseudoClassConditionImpl) condition;
		return c.getValue().matches(this._pseudoClassName.replace("(", "\\(").replace(")", "\\)"));
	}

	@Override
	public String getClassName() {
		return PSEUDO_CLASS_MAPPING.get(this._pseudoClassName);
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) throws IllegalArgumentException {
		throw new RuntimeException();
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		return false;
	}

	public static class PseudoSelector {
		private final String _name; // NOSONAR
		private final boolean _isState; // NOSONAR

		public PseudoSelector(String name, boolean isState) {
			this._name = name;
			this._isState = isState;
		}

		public String getName() {
			return this._name;
		}

		public boolean isState() {
			return this._isState;
		}
	}

	public static Set<String> getAllPseudoClasses() {
		return Collections.unmodifiableSet(PSEUDO_CLASS_MAPPING.keySet());
	}

}
