/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator.global;

import com.is2t.giml.jsgenerator.JavaScriptSourceToMicroEJSourceGenerator;
import com.is2t.giml.jsgenerator.UnsupportedJavaScriptSyntax;
import com.is2t.java.nodes.Expression;
import com.is2t.java.nodes.Statement;

import jdk.nashorn.internal.ir.BlockStatement;
import jdk.nashorn.internal.ir.CallNode;

/**
 * Global object accessed by <console.log(
 */
public interface GlobalObject {

	/**
	 * Get the well-known global that allows to access this object
	 * @return the global name
	 */
	public String getGlobalName();

	/**
	 * Generate the initialization of this global object
	 *
	 * @param gen
	 * @return the initialization statement (may be a {@link BlockStatement} if the
	 *         global object requires mutliple statements)
	 */
	public Statement generateInitialization(JavaScriptSourceToMicroEJSourceGenerator gen);

	/**
	 * Generate a new call expression
	 * 
	 * @return null if the call must be generated in a standard way
	 * @throws UnsupportedJavaScriptSyntax
	 *             if the call could not be generated
	 */
	public Expression generateCallExpression(JavaScriptSourceToMicroEJSourceGenerator gen, CallNode call,
			String functionName);
}
