/**
 * Java
 *
 * Copyright 2011 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.icetea;

import com.is2t.icetea.nodes.AssertStatement;
import com.is2t.icetea.nodes.DirectReadAccessToMemory;
import com.is2t.icetea.nodes.DirectWriteAccessToMemory;
import com.is2t.icetea.nodes.ExternSymbol;
import com.is2t.icetea.nodes.MemoryStartAddress;
import com.is2t.icetea.nodes.StructSizeOf;
import com.is2t.java.JavaSourceToStringGenerator;
import com.is2t.java.nodes.Expression;
import com.is2t.java.nodes.TypeReference;
import com.is2t.java.nodes.expressions.Allocation;
import com.is2t.java.nodes.expressions.CastExpression;
import com.is2t.java.nodes.expressions.IntegerLiteral;
import com.is2t.java.nodes.expressions.MessageSend;
import com.is2t.java.nodes.expressions.ObjectAllocation;

public class IceteaSourceToStringGenerator extends JavaSourceToStringGenerator implements IceteaSourceGenerator{

	public static final String DotIceTea = ".iceTea";
	public static final String Upper = "upper";
	public static final String Ro = "ro";
	public static final String Map = "map";
	public static final String NoInit = "noInit";
	public static final String Alloca = "alloca";
	public static final String Assert = "assert";
	

	private char[] newInlineMethod;
	private char[][] iceteaSupportType;
	
	public IceteaSourceToStringGenerator(){
		newInlineMethod = new char[]{'n','e','w','I','n','l','i','n','e'};
		iceteaSupportType = new char[][]{ new char[]{'i','c','e','T','e','a'},new char[]{'l','a','n','g'},new char[]{'I','c','e','T','e','a','1','_','0','S','u','p','p','o','r','t' }};
	}
	
	protected String getFileExtension() {
		return DotIceTea;
	}
	
	/**
	 * Java has no dimension qualifier
	 */
	public String getDimQualifier(int value){
		if(value == TypeReference.DimQualifierUpper)
			return Upper;
		else if(value == TypeReference.DimQualifierRo)
			return Ro;
		else
			throw new AssertionError();
	}

	public void generateMemoryStartAddress(MemoryStartAddress node) {
		node.receiver.generateExpressionUsing(this);
		stream.append('.');
		stream.append("startAddress");
		if(node.type == TypeReference.T_INT){
			stream.append("Int");
		}
		stream.append("()");
	}

	public void generateStructSizeOf(StructSizeOf node) {
		stream.append("sizeOf(");
		node.type.generateUsing(this);
		stream.append(')');
	}

	public void generateDirectReadAccessToMemory(DirectReadAccessToMemory node) {
		TypeReference t = node.returnType;
		if(t.isDouble() || t.isFloat()){
			// special IceTea1.0 support
			char[] methodName = ("get"+getDirectMemorySuffix(t)).toCharArray();
			new MessageSend(methodName, iceteaSupportType, null, new Expression[]{node.memoryChunk, node.address}).generateExpressionUsing(this);
		}
		else{
			node.memoryChunk.generateExpressionUsing(this);
			stream.append(".get");
			stream.append(getDirectMemorySuffix(node.returnType));
			stream.append('(');
			node.address.generateExpressionUsing(this);
			stream.append(')');
		}
	}
	

	public void generateDirectWriteAccessToMemory(DirectWriteAccessToMemory node) {
		TypeReference t = node.valueType;
		if(t.isDouble() || t.isFloat()){
			// special IceTea1.0 support
			char[] methodName = ("set"+getDirectMemorySuffix(t)).toCharArray();
			new MessageSend(methodName, iceteaSupportType, null, new Expression[]{node.memoryChunk, node.address, node.value}).generateExpressionUsing(this);
		}
		else{
			node.memoryChunk.generateExpressionUsing(this);
			stream.append(".set");
			stream.append(getDirectMemorySuffix(node.valueType));
			stream.append('(');
			node.address.generateExpressionUsing(this);
			stream.append(',');
			node.value.generateExpressionUsing(this);
			stream.append(')');
		}
	}

	private String getDirectMemorySuffix(TypeReference type) {
		String typeGet;
		switch(type.id){
		case TypeReference.T_BYTE:{
			typeGet ="Byte";
			break;
		}
		case TypeReference.T_CHAR:{
			typeGet ="Char";
			break;
		}
		case TypeReference.T_SHORT:{
			typeGet ="Short";
			break;
		}
		case TypeReference.T_INT:{
			typeGet ="Int";
			break;
		}
		case TypeReference.T_LONG:{
			typeGet ="Long";
			break;
		}
		case TypeReference.T_FLOAT:{
			typeGet ="Float";
			break;
		}
		case TypeReference.T_DOUBLE:{
			typeGet ="Double";
			break;
		}
		case TypeReference.T_BOOLEAN:
		case TypeReference.T_REF:
		case TypeReference.T_VOID:
		default:
			throw new AssertionError();
		}
		return typeGet;
	}
	
	protected String getAllocationKeyword(Allocation node) {
		switch(node.getAllocationKind()){
		case Allocation.DefaultAllocation:
			return StrNew;
		case Allocation.Map:
			return Map;
		case Allocation.Alloca:
			return Alloca;
		default:
			throw new AssertionError(); // added a new kind of allocation ?
		}
	}
	

	public void generateExternSymbol(ExternSymbol node) {
		stream.append("IceTea1_0Support").append('.').append("extern");
		if(node.property){
			stream.append("Property");
		}
		stream.append('(').append('\"').append(node.symbol).append('\"').append(')');
	}
	
	public void generateObjectAllocation(ObjectAllocation node) {
		if(node.getAllocationKind() == Allocation.Inline){
			ObjectAllocation map = new ObjectAllocation(node.type, new Expression[]{new IntegerLiteral(0)});
			map.setAllocationKind(Allocation.Map);
			new CastExpression(new MessageSend(newInlineMethod, iceteaSupportType, null, new Expression[]{map}), node.type).generateExpressionUsing(this);
			return;
		}
		super.generateObjectAllocation(node);
	}

	public void generateAssertStatement(AssertStatement node) {
		stream.append(Assert).append(' ');
		node.condition.generateExpressionUsing(this);
		stream.append(Semicolon);
	}

}
