/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.ecom.network.natives;

import ej.ecom.network.IPConfiguration;

/**
 *
 */
public class InterfaceManagerImplNatives {

	/**
	 * Enable an interface.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int enable(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Disable an interface.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int disable(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Tell whether an interface is enabled or not.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return {@link InterfaceManagerImplNativesConstants#IF_STATE_ENABLED} if enabled,
	 *         {@link InterfaceManagerImplNativesConstants#IF_STATE_NOT_ENABLED} if not, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int isEnabled(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Start an interface.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int start(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Stop an interface.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int stop(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Tell whether an interface is started or not.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return {@link InterfaceManagerImplNativesConstants#IF_STATE_STARTED} if started,
	 *         {@link InterfaceManagerImplNativesConstants#IF_STATE_NOT_STARTED} if not, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int isStarted(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Get whether IP configuration is static or not.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return {@link InterfaceManagerImplNativesConstants#IF_IP_STATIC} if true,
	 *         {@link InterfaceManagerImplNativesConstants#IF_IP_DYNAMIC} if provided by DHCP, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int isStatic(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Get the netmask.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer where the address will be stored
	 * @param addressOffset
	 *            the offset where the address will be stored
	 * @param addressLength
	 *            the address buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int getNetmask(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, boolean getResult);

	/**
	 * Get the gateway.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer where the address will be stored
	 * @param addressOffset
	 *            the offset where the address will be stored
	 * @param addressLength
	 *            the address buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int getGateway(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, boolean getResult);

	/**
	 * Get whether DNS is statically configured or given by DHCP.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return {@link InterfaceManagerImplNativesConstants#IF_DNS_STATIC} if true,
	 *         {@link InterfaceManagerImplNativesConstants#IF_DNS_NOT_STATIC} if false, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int isDNSStatic(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Get the number of available DNS servers.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the number of DNS IP address or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int getDNSCount(byte[] netifName, int netifNameOffset, int netifNameLength, boolean getResult);

	/**
	 * Get the address of the n-th DNS server.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer where the address will be stored
	 * @param addressOffset
	 *            the offset where the address will be stored
	 * @param addressLength
	 *            the address buffer length
	 * @param index
	 *            the index of the DNS server (from 0 to {@link #getDNSCount()} -1)
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int getDNS(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, int index, boolean getResult);

	/**
	 * In order to avoid a lot of immortal buffer, parameters are sent one by one to the native world. This method is
	 * here to notify the underlying module that the configuration process will begin. The module may then enter
	 * configuration mode.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int startConfiguration(byte[] netifName, int netifNameOffset, int netifNameLength,
			boolean getResult);

	/**
	 * Ask native to use DHCP.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param use
	 *            true to use DHCP, false otherwise
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int useDHCP(byte[] netifName, int netifNameOffset, int netifNameLength, boolean use,
			boolean getResult);

	/**
	 * Set the IP.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer that store IP address
	 * @param addressOffset
	 *            the offset where the IP address will be stored
	 * @param addressLength
	 *            the IP address buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int setIP(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, boolean getResult);

	/**
	 * Set the netmask.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer that store IP address
	 * @param addressOffset
	 *            the offset where the IP address will be stored
	 * @param addressLength
	 *            the IP address buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int setNetmask(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, boolean getResult);

	/**
	 * Set the gateway.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer that store IP address
	 * @param addressOffset
	 *            the offset where the IP address will be stored
	 * @param addressLength
	 *            the IP address buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int setGateway(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, boolean getResult);

	/**
	 * Ask native to use static DNS.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param use
	 *            true to use static DNS, false otherwise
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int useStaticDNS(byte[] netifName, int netifNameOffset, int netifNameLength, boolean use,
			boolean getResult);

	/**
	 * Set a DNS server. Since an {@link IPConfiguration} contains a list of DNS servers, we need to set them one by one
	 * so that we have only one immortal buffer. Several calls to this native method are made with a index (from 0 to
	 * n-1) to identify the DNS server in the list.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param address
	 *            the buffer that store IP address
	 * @param addressOffset
	 *            the offset where the IP address is stored
	 * @param addressLength
	 *            the IP address buffer length
	 * @param index
	 *            the index of the DNS
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int setDNS(byte[] netifName, int netifNameOffset, int netifNameLength, byte[] address,
			int addressOffset, int addressLength, int index, boolean getResult);

	/**
	 * In order to avoid a lot of immortal buffer, parameters are sent one by one to the native world. This method is
	 * here to apply the previously set parameters. Some modules need to reboot to apply the IP configuration; some
	 * others can apply on the fly and won't do anything in this method.
	 *
	 * @param netifName
	 *            the buffer that store interface name
	 * @param netifNameOffset
	 *            the offset where the interface name is stored
	 * @param netifNameLength
	 *            the interface name buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 on success or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int endConfiguration(byte[] netifName, int netifNameOffset, int netifNameLength,
			boolean getResult);

}
